package dev.peerat.parser.java.tree;

import dev.peerat.parser.Token;
import dev.peerat.parser.TokenType;
import dev.peerat.parser.java.Annotation.AnnotableBuffer;
import dev.peerat.parser.java.ClassContainer;
import dev.peerat.parser.java.Enumeration;
import dev.peerat.parser.java.JavaElement;
import dev.peerat.parser.state.RedirectStateTree;
import dev.peerat.parser.state.StateTree;
import dev.peerat.parser.tree.SyntaxTree;
import dev.peerat.parser.tree.SyntaxTreeRegistery;

public class EnumTree extends SyntaxTree<JavaElement> {

	public EnumTree(SyntaxTreeRegistery<JavaElement> registery) {
		super(registery);
		this.current = new StateTree<>();
	}

	@Override
	public void init() {
		StateTree<JavaElement> modifier = registery.getNode(ModifierTree.class);
		StateTree<JavaElement> type = registery.getNode(TypeTree.class);
		StateTree<JavaElement> enum_value = registery.getNode(EnumValueTree.class);
		StateTree<JavaElement> clazz_ = registery.getNode(ClassBaseTree.class);
		// ENUM
		StateTree<JavaElement> enums = this.current;
		StateTree<JavaElement> enum_base = enums
				.then((validator) -> validator.validate((token) -> token.getValue().equals("enum")))
				.then((validator) -> validator.validate((token) -> token.getType().equals(TokenType.NAME),
						(bag, token) -> bag.set("name", token)));

		StateTree<JavaElement> enum_start = enum_base
				.then((validator) -> validator.validate((token) -> token.getValue().equals("{")));
		enum_start.<JavaElement>end((parent, bag) -> {
			Enumeration current = new Enumeration((((AnnotableBuffer) parent).getAnnotationBuffer()), bag.get("name"),
					bag.get("implement"));
			if (parent instanceof ClassContainer)
				((ClassContainer) parent).addClass(current);
			return current;
		}).unique(enum_value)
//					.unique(mult_clazz_container)
				.unique((validator) -> validator.validate((token) -> token.getValue().equals("}"))).end((a, b) -> a);

		StateTree<JavaElement> enum_implement = enum_base
				.then((validator) -> validator.validate((token) -> token.getValue().equals("implements")));
		StateTree<JavaElement> enum_implement_name = enum_implement
				.then(new RedirectStateTree<>(type, (global, local) -> {
					Token token = global.get("implement");
					if (token == null)
						token = local.get();
					else
						token = token.concat(local.get());
					global.set("implement", token);
				}));
		enum_implement_name.then((validator) -> validator.validate((token) -> token.getValue().equals(",")))
				.then(enum_implement_name);
		enum_implement_name.then(enum_start);
		StateTree<JavaElement> enum_extend = enum_base
				.then((validator) -> validator.validate((token) -> token.getValue().equals("extends")))
				.then(new RedirectStateTree<>(type, (global, local) -> global.set("extend", local.get())));
		enum_extend.then(enum_start);
		enum_extend.then(enum_implement);

		StateTree<JavaElement> enum_mod = clazz_
				.then(new RedirectStateTree<>(modifier, (global, local) -> global.set("modifier", local)));
		clazz_.then(enums);
		enum_mod.then(enums);
	}

}
