package dev.peerat.parser.java.visitor;

import java.util.function.Predicate;

import dev.peerat.parser.java.Variable;
import dev.peerat.parser.visitor.VisitorBag;

public class JavaVariableVisitor extends JavaVisitor<Variable>{
	
	private Predicate<String> typeFilter;
	private Predicate<String> nameFilter;
	private Predicate<Integer> modifierFilter;
	
	public JavaVariableVisitor(){
		super(Variable.class);
	}
	
	public JavaVariableVisitor typeFilter(Predicate<String> validator){
		this.typeFilter = (this.typeFilter == null) ? validator : this.typeFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor nameFilter(Predicate<String> validator){
		this.nameFilter = (this.nameFilter == null) ? validator : this.nameFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isPublic(){
		Predicate<Integer> validator = (mod) -> (mod & 0x1) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isPrivate(){
		Predicate<Integer> validator = (mod) -> (mod & 0x2) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isProtected(){
		Predicate<Integer> validator = (mod) -> (mod & 0x4) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isStatic(){
		Predicate<Integer> validator = (mod) -> (mod & 0x8) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isFinal(){
		Predicate<Integer> validator = (mod) -> (mod & 0x10) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isVolatile(){
		Predicate<Integer> validator = (mod) -> (mod & 0x40) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isTransient(){
		Predicate<Integer> validator = (mod) -> (mod & 0x80) != 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}
	
	public JavaVariableVisitor isPackageLevel(){
		Predicate<Integer> validator = (mod) -> (mod & 0x1) == 0 && (mod & 0x2) == 0 && (mod & 0x4) == 0;
		this.modifierFilter = (this.modifierFilter == null) ? validator : this.modifierFilter.and(validator);
		return this;
	}

	@Override
	public VisitorBag visitElement(Variable element){
		VisitorBag bag = new VisitorBag();
		
		if(typeFilter != null){
			//ellipse ?
			if(!typeFilter.test(element.getType().getValue())) return bag;
		}
		
		if(nameFilter != null){
			if(!nameFilter.test(element.getName().getValue())) return bag;
		}
		
		if(this.modifierFilter != null){
			if(!this.modifierFilter.test(element.getModifier())) return bag;
		}
		
		bag.compute(element);
		return bag;
	}
	
	

}
