package dev.peerat.parser.state;

import java.util.function.BiConsumer;

import dev.peerat.parser.Bag;
import dev.peerat.parser.TokenValidator;

public class RedirectStateTree<E, T extends E> extends StateTree<T>{
	
	private StateTree<E> redirect;
	private BiConsumer<Bag, Bag> group;
	
	private StackTraceElement calling;

	public RedirectStateTree(StateTree<E> redirect, BiConsumer<Bag, Bag> group){
		super();
		this.redirect = redirect;
		this.group = group;
		
		StackTraceElement[] trace = Thread.currentThread().getStackTrace();
		int current = 0;
		while((current < trace.length) && (!(trace[++current].getClassName().contains("Value"))));
		
		this.calling = trace[current];
	}
	
	@Override
	BuilderStateTree<T, ?> internalSeed(TokenValidator validator, T element){
		System.out.println("[redirect into] "+calling);
		Bag currentBag = validator.getBag();
		Bag localBag = new Bag();
		
		TokenValidator branch = validator.branch();
		branch.setBag(localBag);
		BuilderStateTree<E, ?> builded = redirect.internalSeed(branch, (E) element);
		if(builded == null) return null;
		builded.build(branch, element);
		
		System.out.println("[redirect done] "+calling);

		this.group.accept(currentBag, localBag);
		branch.setBag(currentBag);
		validator.merge(branch);
		
		return super.internalSeed(validator, element);
	}
}

