package dev.peerat.parser.state;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;
import java.util.function.BiFunction;
import java.util.function.Function;

import dev.peerat.parser.Bag;
import dev.peerat.parser.Token;
import dev.peerat.parser.TokenValidator;

public class StateTree<E>{
	
	private List<StateTree<E>> childs;
	
	Function<TokenValidator, Boolean> checker;
	private BuilderStateTree<E, ?> builder;
	
	private StackTraceElement origin;
	
	//TODO checker in constructeur ?
	public StateTree(){
		this.childs = new ArrayList<>();
	}
	
	private void origin(){
		StackTraceElement[] trace = Thread.currentThread().getStackTrace();
		int current = 0;
		while((current < trace.length) && (!(trace[++current].getClassName().contains("Value"))));
		
		this.origin = trace[current];
	}
	
	public void seed(TokenValidator validator, E container){
		while(validator.hasNext()){
			BuilderStateTree<E, ?> build = internalSeed(validator, container);
			if(build == null) break;
			build.build(validator, container);
		}
	}
	
	BuilderStateTree<E, ?> internalSeed(TokenValidator validator, E element){
		System.out.println("internalSeed "+origin);
		for(StateTree<E> child : this.childs){
			System.out.println("internalSeed "+origin+" child: "+child.origin);
			TokenValidator branch = validator.branch();
			if(child.checker == null){
				System.out.println("checker null");
				BuilderStateTree<E, ?> builded = child.internalSeed(branch, element);
				if(builded != null){
					validator.merge(branch);
					return builded;
				}
				continue;
			}
			System.out.println("try checker");
			if(child.checker.apply(branch)){
				System.out.println("pass");
				BuilderStateTree<E, ?> builded = child.internalSeed(branch, element);
				if(builded != null){
					validator.merge(branch);
					return builded;
				}
			}
		}
		if(this.builder != null) return this.builder;
		return null;
	}

	public <T extends StateTree<E>> StateTree<E> then(StateTree<E> child){
		childs.add(child);
		return child;
	}
	
	public <T extends StateTree<E>> StateTree<E> then(Function<TokenValidator, Boolean> checker){
		StateTree<E> state = new StateTree<>();
		state.checker = checker;
		this.childs.add(state);
		state.origin();
		return state;
	}
	
	public <T extends StateTree<E>> StateTree<E> loop(){
		this.childs.add(this);
		return this;
	}
	
	public <B> BuilderStateTree<E, B> end(){
		return end((a,b) -> null);
	}
	
	public <B> BuilderStateTree<E, B> end(BiFunction<E, Bag, B> builder){
		BuilderStateTree<E, B> builderState = new BuilderStateTree<>(builder);
		this.builder = builderState;
		return builderState;
	}
	
	//maybe remove later
	public StateTree<E> equals(String value){
		return then((validator) -> validator.validate((token) -> token.getValue().equals(value)));
	}
	
	public StateTree<E> equals(String value, String key){
		return then((validator) -> validator.validate((token) -> token.getValue().equals(value), (bag, token) -> bag.set(key, token)));
	}
	
	public StateTree<E> equals(String value, String key, String val){
		return then((validator) -> validator.validate((token) -> token.getValue().equals(value), (bag, token) -> bag.set(key, val)));
	}
	
	public StateTree<E> equals(String value, BiConsumer<Bag, Token> consumer){
		return then((validator) -> validator.validate((token) -> token.getValue().equals(value), consumer));
	}
	
	public StateTree<E> redirect(StateTree<E> target){
		return then(new RedirectStateTree<>(target, (global, local) -> {
			System.out.println("\nMERGE:\n\t"+global+"\n\t"+local+"\n");
			global.set(local.get());
		}));
	}
	
	public StateTree<E> redirect(StateTree<E> target, BiConsumer<Bag, Bag> group){
		return then(new RedirectStateTree<>(target, group));
	}
}
